%%  Author: Panos N. Alevizos
%
%   Ph.D. Student and Research Assistant
%   School of Electrical and Computer Engineering
%   Technical University of Crete
%   Email: palevizos@isc.tuc.gr or bigpan88@hotmail.com
%   Webpage: http://users.isc.tuc.gr/~palevizos/
%
%   Date: January 2018
%
%   References:
%   P. N. Alevizos, K. Tountas, and A. Bletsas, "Multistatic Scatter Radio Sensor Networks for Extended Coverage," IEEE Transactions On Wireless Communications,
%   Volume: XX, Issue: XX, pp. XX--XX, 2018.
%
%   **Inquiries regarding the script provided below are cordially welcome.
%   In case you spot a bug, please let me know.
%   If you use some piece of code for your own work, please cite the
%   article above.**
%
%% this code is for Fig. 5 of the paper

%% Reset.
close all;
clear all;
clc;

%%   Propagation and channel variance and noise Parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%noise figure (dB)
noiseFigure = 5;
%noise energy (dBm/Hz)
N0_dB = -174 + noiseFigure;
%Noise energy (Watt/Hz)
N0 = 0.001*10^(N0_dB/10);
%F carier
F_carrier = 0.868 * 10^9;
%propagation wavelength (meters)
lambda = 3*10^8 / F_carrier;
%reference distance
d_ref = 1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%              Tag Parameters.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Tag data rate (bps).
rate = 1e3;                     
%bit duration
T = 1/rate;                      
%Tag scattering efficiency.
scatter_eff = 1e-1;             
%antenna load values
Gamma_0 = 1;
Gamma_1 = -1;
Gamma_0_1 = abs(Gamma_0 - Gamma_1);
%one tag (and also one slot) 
N = 1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%






%%     topology parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% size of the topology (m)
M=40;
%Reader and carrier emitter position for monostatic and bistatic system
u_SDRReader= [0;0];
u_CE = [M; M;];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%       random experiment monte Carlo parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%power tx in dBm
P_tx_dBm = 0 : 3 : 30;


%maximum number of iterations
MaxItersMonteCarlo = 3000;

%number of channel realizations
Max_Chann_Real = 200000;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% BER expressions
theory_mon_1_coh = zeros(size(P_tx_dBm));
theory_bis_1_coh  = zeros(size(P_tx_dBm));
theory_mon_1_NC = zeros(size(P_tx_dBm));
theory_bis_1_NC  = zeros(size(P_tx_dBm));
P_e_upper_bound_mon  = zeros(size(P_tx_dBm));
P_e_upper_bound_bis  = zeros(size(P_tx_dBm));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for P_tx_iter = 1 : length(P_tx_dBm)
    
    P_tx_iter
    
    %current value of P_tx in Watts
    P_tx_cur = 0.001*10^(P_tx_dBm(P_tx_iter)/10);
    
    
    for iter_out = 1 : MaxItersMonteCarlo
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %parameters for Nakagami (Rician --> Nakagami)
        kappa_n = 20*rand;
        kappa_ln = 20*rand;
        M_n = (kappa_n+1)^2 / (2*kappa_n+1);
        M_ln = (kappa_ln+1)^2 / (2*kappa_ln+1);
        pd_CT = makedist('Nakagami','mu',M_ln,'omega',1); %create random objects
        pd_TR = makedist('Nakagami','mu',M_n,'omega',1); %create random objects
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        

        
        %% PLEs generation
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Path-loss exponents for monostatic system
        PLE_mon_TR = 2 + 0.5*rand(N,1);
        
        % Path-loss exponents for bistatic system
        PLE_bis_TR = PLE_mon_TR;
        PLE_bis_CT = 2 + 0.5*rand(N,1);
        PLE_bis_CT_vec = PLE_bis_CT(:);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        
        
        %% position of tag
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %tag positions for the current experiment  
        u_TAGS = [M*rand; M*rand];
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        
        
        %% create experiment for  monostatic system
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %distance between tags and SDR reader
        d_TR_all_mon = pdist2(u_TAGS', u_SDRReader');
        
        %one-way path gain for TR link for monostatic system
        L_TR_all_mon = ((lambda./(4*pi*d_ref)).^2).*((d_ref./d_TR_all_mon).^PLE_mon_TR);
        
        
        %\mu for monostatic (Eq. 15)
        mu_mon = sqrt(2 * P_tx_cur) *  L_TR_all_mon * Gamma_0_1 * (2/pi) * scatter_eff ;
        %Average received energy for monostatic (Eq. 14)
        E_mon = 0.5 * ( (M_n + 1)./ ((M_n )) ) .* T .* (mu_mon).^2;
        
        %average received SNR for monostatic system
        SNR_mon = E_mon  /N0;
        
        %coherent reception
        theory_mon_1_coh(P_tx_iter) = theory_mon_1_coh(P_tx_iter) + mean(qfunc(sqrt(SNR_mon*M_n/(M_n+1))  * random(pd_TR,Max_Chann_Real,1).^2));
        
        %noncoherent reception
        theory_mon_1_NC(P_tx_iter) = theory_mon_1_NC(P_tx_iter) + 0.5*mean(exp(-0.5*(SNR_mon*M_n/(M_n+1))  * random(pd_TR,Max_Chann_Real,1).^4));
        
        
        %derived expression for monostatic (Eq. 18)
        z_mon = (M_n + M_n^2) ./ (2 * SNR_mon);
        P_e_upper_bound_mon(P_tx_iter) = P_e_upper_bound_mon(P_tx_iter) + 0.5* z_mon.^(M_n/2) .* myKummerU(M_n/2,1/2,z_mon);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        
        
        
        %% create experiment for  bistatic system
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %distance between tag and carrier emitter
        d_CT_all_bis = pdist2(u_TAGS', u_CE');
        
        %distance between tag and SDR reader
        d_TR_all_bis = pdist2(u_TAGS', u_SDRReader');
        
        %one-way path gain for CT link for bistatic system
        L_CT_all_bis = ((lambda./(4*pi*d_ref)).^2).*((d_ref./d_CT_all_bis).^PLE_bis_CT) ;
        
        %one-way path gain for TR link for bistatic system
        L_TR_all_bis = ((lambda./(4*pi*d_ref)).^2).*((d_ref./d_TR_all_bis).^PLE_bis_TR);
        
        
        %\mu for bistatic (Eq. 10)
        mu_bis = sqrt(2* P_tx_cur) *  sqrt(L_CT_all_bis.*L_TR_all_bis)  * Gamma_0_1 * (2/pi) * scatter_eff;
        %Average received energy for monostatic (Eq. 9)
        E_bis = 0.5 * T * (mu_bis).^2;
        
        %average received SNR for bistatic system
        SNR_bis = E_bis   /N0;
        
        %coherent reception
        theory_bis_1_coh(P_tx_iter) =theory_bis_1_coh(P_tx_iter) +  mean(qfunc(sqrt(SNR_bis)  *...
            random(pd_TR,Max_Chann_Real,1).*random(pd_CT,Max_Chann_Real,1)));
        
        %noncoherent reception
        theory_bis_1_NC(P_tx_iter) =theory_bis_1_NC(P_tx_iter) +  0.5*mean(exp(-0.5*(SNR_bis)  *...
            random(pd_TR,Max_Chann_Real,1).^2 .* random(pd_CT,Max_Chann_Real,1).^2));
        
        %derived expression for bistatic (Eq. 21)
        z_bis = 2*M_ln*M_n ./ SNR_bis;
        P_e_upper_bound_bis(P_tx_iter) = P_e_upper_bound_bis(P_tx_iter) + 0.5* z_bis.^(M_n) .* myKummerU(M_n, 1 + M_n - M_ln,z_bis);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    end
    
    theory_mon_1_coh(P_tx_iter) =theory_mon_1_coh(P_tx_iter)/MaxItersMonteCarlo;
    theory_mon_1_NC(P_tx_iter) =theory_mon_1_NC(P_tx_iter)/MaxItersMonteCarlo;    
    P_e_upper_bound_mon(P_tx_iter) = P_e_upper_bound_mon(P_tx_iter)/MaxItersMonteCarlo;
    
    theory_bis_1_coh(P_tx_iter) =theory_bis_1_coh(P_tx_iter)/MaxItersMonteCarlo;
    theory_bis_1_NC(P_tx_iter) =theory_bis_1_NC(P_tx_iter)/MaxItersMonteCarlo;    
    P_e_upper_bound_bis(P_tx_iter) = P_e_upper_bound_bis(P_tx_iter)/MaxItersMonteCarlo;
    
end

